"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractAmqpConsumer = void 0;
const AbstractAmqpService_1 = require("./AbstractAmqpService");
const amqpMessageDeserializer_1 = require("./amqpMessageDeserializer");
const amqpErrors_1 = require("./errors/amqpErrors");
const ABORT_EARLY_EITHER = {
    error: 'abort',
};
class AbstractAmqpConsumer extends AbstractAmqpService_1.AbstractAmqpService {
    transactionObservabilityManager;
    errorResolver;
    constructor(dependencies, options) {
        super(dependencies, options);
        this.transactionObservabilityManager = dependencies.transactionObservabilityManager;
        this.errorResolver = dependencies.consumerErrorResolver;
    }
    deserializeMessage(message) {
        if (message === null) {
            return ABORT_EARLY_EITHER;
        }
        const deserializationResult = (0, amqpMessageDeserializer_1.deserializeMessage)(message, this.messageSchema, this.errorResolver);
        if (deserializationResult.error instanceof amqpErrors_1.AmqpValidationError ||
            deserializationResult.error instanceof amqpErrors_1.AmqpMessageInvalidFormat) {
            this.handleError(deserializationResult.error);
            return ABORT_EARLY_EITHER;
        }
        // Empty content for whatever reason
        if (!deserializationResult.result) {
            return ABORT_EARLY_EITHER;
        }
        return {
            result: deserializationResult.result,
        };
    }
    async start() {
        await this.init();
        if (!this.channel) {
            throw new Error('Channel is not set');
        }
        await this.channel.consume(this.queueName, (message) => {
            if (message === null) {
                return;
            }
            const deserializedMessage = this.deserializeMessage(message);
            if (deserializedMessage.error === 'abort') {
                this.channel.nack(message, false, false);
                return;
            }
            const transactionSpanId = `queue_${this.queueName}:${
            // @ts-ignore
            // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
            deserializedMessage.result[this.messageTypeField]}`;
            this.transactionObservabilityManager?.start(transactionSpanId);
            this.processMessage(deserializedMessage.result)
                .then((result) => {
                if (result.error === 'retryLater') {
                    this.channel.nack(message, false, true);
                }
                if (result.result === 'success') {
                    this.channel.ack(message);
                }
            })
                .catch((err) => {
                // ToDo we need sanity check to stop trying at some point, perhaps some kind of Redis counter
                // If we fail due to unknown reason, let's retry
                this.channel.nack(message, false, true);
                this.handleError(err);
            })
                .finally(() => {
                this.transactionObservabilityManager?.stop(transactionSpanId);
            });
        });
    }
}
exports.AbstractAmqpConsumer = AbstractAmqpConsumer;
//# sourceMappingURL=AbstractAmqpConsumer.js.map